/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package fi.cgi.bis.soa.example;

import java.io.*;
import javax.xml.XMLConstants;

import org.apache.commons.io.LineIterator;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.milyn.cdr.annotation.AppContext;
import org.milyn.cdr.annotation.ConfigParam;
import org.milyn.container.ApplicationContext;
import org.milyn.container.ExecutionContext;
import org.milyn.delivery.annotation.Initialize;
import org.milyn.javabean.repository.BeanId;
import org.milyn.xml.SmooksXMLReader;
import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;

/**
 * Splits finvoice files to and publishes contents of the file as FinvoiceDocument-element
 *
 * @author paloniemit
 */
public class FinvoiceSplittingReader implements SmooksXMLReader {
    
	private static final Logger log = Logger.getLogger(FinvoiceSplittingReader.class);
	
    public static String FILE_ELEMENT_NAME = "FinvoiceDocument";

    @ConfigParam(name="beanId")
    private String beanIdString;
    private BeanId beanId;
    private ContentHandler contentHandler;
    private ExecutionContext context;

    @AppContext
    private ApplicationContext applicationContext;

    @Initialize
    public void init(){
        if(beanIdString != null){
            beanId = applicationContext.getBeanIdStore().register(beanIdString);
        }

    }

    @Override
    public void setExecutionContext(ExecutionContext executionContext) {
        this.context = executionContext;
    }

    @Override
    public boolean getFeature(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setFeature(String name, boolean value) throws SAXNotRecognizedException, SAXNotSupportedException {
    }

    @Override
    public Object getProperty(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setProperty(String name, Object value) throws SAXNotRecognizedException, SAXNotSupportedException {
    }

    @Override
    public void setEntityResolver(EntityResolver resolver) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public EntityResolver getEntityResolver() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setDTDHandler(DTDHandler handler) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public DTDHandler getDTDHandler() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setContentHandler(ContentHandler handler) {
        this.contentHandler = handler;
    }

    @Override
    public ContentHandler getContentHandler() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setErrorHandler(ErrorHandler handler) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public ErrorHandler getErrorHandler() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void parse(InputSource input) throws IOException, SAXException {
        contentHandler.startDocument();
        processFileContent(getReader(input));
        contentHandler.endDocument();
    }

    private Reader getReader(InputSource input) throws UnsupportedEncodingException {
        BufferedReader reader = null;
        if(input.getCharacterStream() != null){
            log.debug("Reading finvoice from characterStream.");
            return new BufferedReader(input.getCharacterStream());
        }else if(input.getByteStream() != null){
            log.debug("Reading finvoice from bytestream with context encoding: " + context.getContentEncoding());
            return new BufferedReader(new InputStreamReader(input.getByteStream(), context.getContentEncoding()));
        }
        throw new RuntimeException("Could not read from InputSource");
    }

    private void processFileContent(Reader reader) throws SAXException {
        boolean finvoiceElementOpen = false;
        LineIterator it = new LineIterator(reader);
        // I build the document as a string to a bean because for some reason reading from FinvoiceDocument escapes the
        // data using xml entities. If you can solve this then you can remove the StringBuilder stuff
        StringBuilder documentContentBuilder = new StringBuilder();
        while(it.hasNext()){
            String line = it.nextLine();
            if(!finvoiceElementOpen){
                if(StringUtils.isBlank(line)){
                    continue;
                }
                contentHandler.startElement(XMLConstants.NULL_NS_URI, FILE_ELEMENT_NAME, "", new AttributesImpl());
                finvoiceElementOpen = true;
                documentContentBuilder = new StringBuilder();
            }
            char[] lineChars = line.toCharArray();
            lineChars = ArrayUtils.add(lineChars, '\n');
            contentHandler.characters(lineChars, 0, lineChars.length);
            documentContentBuilder.append(lineChars);
            if(line.startsWith("</Finvoice>")){
                if(beanId != null){
                    context.getBeanContext().addBean(beanId, documentContentBuilder.toString());
                }
                contentHandler.endElement(XMLConstants.NULL_NS_URI, FILE_ELEMENT_NAME, "");
                finvoiceElementOpen = false;
            }
        }
        if(finvoiceElementOpen){
            contentHandler.endElement(XMLConstants.NULL_NS_URI, FILE_ELEMENT_NAME, "");
        }
    }

    @Override
    public void parse(String systemId) throws IOException, SAXException {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
}        

